/*

*********************************************************
/Parametric Soft Robot Mold Generator/ 
by
Jonas Jørgensen (jjoe@itu.dk)

[CopyFarLeft: This code may be copied and used in any 
altered and unaltered form freely as long as it is not 
done to make money.]

Please attribute the author in your projects and feel 
free to notify me if this has been of any use to you :-)
*********************************************************

*/
$fn=30;

//////////////////////////////////////////////////////////////////////////////////////////////////
///***CHANGE THESE PARAMETERS TO ALTER THE SOFT ROBOT'S MORPHOLOGY!***////////////////////////////
Air_channel_length = 110; //length including connector piece 
Air_connector_length = 10; //length of the connector piece without ribs and air channel
Air_channel_width = 3;
Air_channel_height = 4;//Rib_element_height-1; //Must be less than or equal to Rib_element_height

Rib_element_width = 18;
Rib_element_depth = 1.5;
Rib_element_height = 9;
Rib_element_spacing = 3.5;

Robot_endwall_thickness=2;
Robot_sidewall_thickness=3;
Robot_topwall_thickness=2.5;

TwistAngle=0; //Angle to rotate the inner ribs
Air_channel_pos_offset = 0;//Rib_element_width/5;
//Air_channel_pos_offset = -3*Air_channel_width/4;//Air channel position offset distance relative to middle of mold

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////

//++++++++++++++++++++++++++++++++++++++++NEW PARAMETERS:++++++++++++++++++++++++++++++++++++++++//
SlantDeg=0;//Degrees the mold bottom is slanting
RibAttFunction=0; //Attenuation/dampening function for the width of the ribs towards the end of limb (0=none, 1=linear, 2=exp, ...)
RibVarFunction = 0; //Periodical function to vary the width of the ribs (0=none, 1=sine, 2=saw.., 3=sqaure...)
WallAttFunction=0; //Attenuation/dampening function for the outline of the mold walls towards the end of limb (0=none, 1=linear, 2=exp, ...)
WallVarFunction = 0; //Periodical function to vary the outline of the mold walls (0=none, 1=sine, 2=saw.., 3=sqaure...)
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//


Mold_bottom_thickness=3;
Mold_wall_thickness=2;
Mold_length = Air_channel_length+Mold_wall_thickness*2+Robot_endwall_thickness*2;
Mold_width=Rib_element_width*cos(TwistAngle)+Mold_wall_thickness*2+Robot_sidewall_thickness*2; 
Mold_wall_height=Rib_element_height+Robot_topwall_thickness+1; //Mold is to be filled 1mm below surface

Number_of_ribs=((Air_channel_length-Air_connector_length)/(Rib_element_depth+Rib_element_spacing))+1;


union() {

    
 //INNERS 
    


    
difference() {

    linear_extrude(height = Rib_element_height, center = true, convexity = 10, slices = 20, scale = 1.0) {
    for (i = [1:Number_of_ribs]){
    translate([0, Rib_element_spacing+(Rib_element_depth/2)+(i-1)*(Rib_element_depth+Rib_element_spacing),0]) rotate(a = TwistAngle, v = [0, 0, 1]) square(size = [Rib_element_width, Rib_element_depth], center = true); //Ribbing elements (The placement of these is used as a basis for calculating the other positions)
    }
    
    }

    
//Cut slanting ribs away at the ends inside the mold:

translate([-(Mold_width-Mold_wall_thickness*2)/2, -Robot_endwall_thickness-(Rib_element_depth/2),-Mold_wall_height/2]) cube([Mold_width-Mold_wall_thickness*2,Robot_endwall_thickness,Mold_wall_height]); 
translate([-(Mold_width-Mold_wall_thickness*2)/2, Air_channel_length+(Rib_element_depth/2),-Mold_wall_height/2]) cube([Mold_width-Mold_wall_thickness*2,Robot_endwall_thickness,Mold_wall_height]);

translate([-(Mold_width-Mold_wall_thickness*2)/2, Air_channel_length-Air_connector_length,-Mold_wall_height/2]) cube([Mold_width-Mold_wall_thickness*2,Air_connector_length,Mold_wall_height]);//cut off ribs at air connector piece
    
translate([-(Mold_width-Mold_wall_thickness*2)/2, -Mold_wall_thickness-Rib_element_width-Robot_endwall_thickness,-Mold_wall_height/2]) cube([Mold_width-Mold_wall_thickness*2,Rib_element_width,Mold_wall_height]);//In front of mold cut off 
translate([-(Mold_width-Mold_wall_thickness*2)/2, Rib_element_width+Mold_length-Mold_wall_thickness-Rib_element_width-Robot_endwall_thickness,-Mold_wall_height/2]) cube([Mold_width-Mold_wall_thickness*2,Rib_element_width,Mold_wall_height]);//Behind mold cut off 


    }
    
    
translate([Air_channel_pos_offset, Air_channel_length/2-Air_connector_length/2,(Air_channel_height/2)-(Rib_element_height/2)]) cube([Air_channel_width, Air_channel_length-Air_connector_length, Air_channel_height], center = true); //Air channel


    
//OUTER MOLD WALLS
translate([0, Air_channel_length/2,((Mold_wall_height-Rib_element_height)/2)-Mold_bottom_thickness/2]) linear_extrude(height = (Mold_bottom_thickness+Mold_wall_height), center = true, convexity = 10, slices = 20, scale = 1.0) {
    
    difference(){
    square(size = [Mold_width, Mold_length], center = true); //Mold (complete volume)
    square(size = [Mold_width-Mold_wall_thickness*2, Mold_length-Mold_wall_thickness*2], center = true); //Mold (inner volume)
    }

    }

 //BOTTOM OF MOLD
 
 translate([0, Air_channel_length/2,(-Rib_element_height/2)-Mold_bottom_thickness/2]) linear_extrude(height = Mold_bottom_thickness, center = true, convexity = 10, slices = 20, scale = 1.0) {
    
    square(size = [Mold_width, Mold_length], center = true); //Mold (complete volume)

    }

//translate([0, 15,-2.5])  cube([15,30, 1.5], center=true);//Bottom
//translate([0, 15,-2.5])  cube([1.5,30, 10], center=true);//Right side
    
}